VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "SqlBuilder"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'Simple class for building SQL statements dynamically
'(c)2003 Yuri Volkov, programmer@iname.com
'
' Select <SelectList> [<set quantifier>] [TOP <top>] [From <From>]
' [Where <Where>]
' [Group By <GroupBy>] [Having <Having>] [Order By <OrderBy>]
' VB6 / MSAccess version
' v.1.0.0 2003-09-30
Option Explicit

Private mIsDistinct As Boolean
Private mTop As String
Private mSelectList As String
Private mFrom As String
Private mWhere As String
Private mGroupBy As String
Private mHaving As String
Private mOrderBy As String

'<set quantifier> is DISTINCT
Public Property Get IsDistinct() As Boolean
  IsDistinct = mIsDistinct
End Property

Public Property Let IsDistinct(ByVal NewValue As Boolean)
  mIsDistinct = NewValue
End Property

'Note: TOP predicate is not standard SQL solution.
'It works for Microsoft products (SQL Server, MS Access).
'For code portability you should use this approach instead:
' select  a.ordered_column, ...
' from table1 a
' where 10 >(select count(1)
'                 from table1 b
'                 where b.ordered_column < a.ordered_column)
' order by a.ordered_column
'
'The above example is equivalent to "TOP 10".
Public Property Get Top() As String
  Top = mTop
End Property

Public Property Let Top(ByVal NewValue As String)
  mTop = NewValue
End Property

'select list
Public Property Get SelectList() As String
  SelectList = mSelectList
End Property

Public Property Let SelectList(ByVal NewValue As String)
  mSelectList = NewValue
End Property

'from clause
Public Property Get From() As String
  From = mFrom
End Property

Public Property Let From(ByVal NewValue As String)
  mFrom = NewValue
End Property

'where clause
Public Property Get Where() As String
  Where = mWhere
End Property

Public Property Let Where(ByVal NewValue As String)
  mWhere = NewValue
End Property

'group by clause
Public Property Get GroupBy() As String
  GroupBy = mGroupBy
End Property

Public Property Let GroupBy(ByVal NewValue As String)
  mGroupBy = NewValue
End Property

'having clause
Public Property Get Having() As String
  Having = mHaving
End Property

Public Property Let Having(ByVal NewValue As String)
  mHaving = NewValue
End Property

'order by clause
Public Property Get OrderBy() As String
  OrderBy = mOrderBy
End Property

Public Property Let OrderBy(ByVal NewValue As String)
  mOrderBy = NewValue
End Property

'append one or more columns to the <select list> clause
Public Function AppendSelectList(ByVal strValue As String) As SqlBuilder
  If Trim(strValue) = "" Then GoTo ExitProc
  If mSelectList <> "" Then
    mSelectList = mSelectList & ", "
  End If
  mSelectList = mSelectList & Trim(strValue)
ExitProc:
  Set AppendSelectList = Me
  Exit Function
End Function

Function AppendFrom(ByVal strValue As String) As SqlBuilder
Dim lng1 As Long
  If Trim(strValue) = "" Then GoTo ExitProc
  If mFrom <> "" Then
    'find "join" word
    lng1 = InStr(1, mFrom, "join", vbTextCompare)
    If lng1 > 2 And Len(mFrom) > lng1 + 4 Then
      If Asc(Mid(mFrom, lng1 - 1, 1)) <= 32 And Asc(Mid(mFrom, lng1 + 4, 1)) <= 32 Then
        mFrom = "(" & mFrom & ")"
      End If
    End If
    mFrom = mFrom & " "
  End If
  mFrom = mFrom & Trim(strValue)

ExitProc:
  Set AppendFrom = Me
  Exit Function
End Function

Public Function AppendWhere(ByVal strValue As String, Optional ByVal blnOr As Boolean = False) As SqlBuilder
  If Trim(strValue) = "" Then GoTo ExitProc
  If mWhere <> "" Then
    If blnOr Or Mid(mWhere, 1, 1) <> "(" Then
      mWhere = "(" & mWhere & ")"
    End If
    If blnOr Then
      mWhere = mWhere & " Or "
    Else
      mWhere = mWhere & " And "
    End If
  End If
  mWhere = mWhere & "(" & Trim(strValue) & ")"

ExitProc:
  Set AppendWhere = Me
  Exit Function
End Function

Public Function AppendOrderBy(ByRef mOrderBy As String, ByVal strValue As String) As SqlBuilder
  If Trim(strValue) = "" Then GoTo ExitProc
  If mOrderBy <> "" Then
    mOrderBy = mOrderBy & ", "
  End If
  mOrderBy = mOrderBy & Trim(strValue)

ExitProc:
  Set AppendOrderBy = Me
  Exit Function
End Function

'Building resulted SQL statement
Public Function ToString() As String
Dim strSql As String

  strSql = "Select"
  If IsDistinct Then
    strSql = strSql & " Distinct"
  End If
  If Top <> "" Then
    strSql = strSql & " Top " & Top
  End If
  If SelectList <> "" Then
    strSql = strSql & " " & SelectList
  Else
    strSql = strSql & " *"
  End If
  If From <> "" Then
    strSql = strSql & " From " & From
  End If
  If Where <> "" Then
    strSql = strSql & " Where " & Where
  End If
  If GroupBy <> "" Then
    strSql = strSql & " Group By " & GroupBy
  End If
  If Having <> "" Then
    strSql = strSql & " Having " & Having
  End If
  If OrderBy <> "" Then
    strSql = strSql & " Order By " & OrderBy
  End If

ExitProc:
  ToString = strSql
  Exit Function
End Function

Public Sub Clear()
  On Error Resume Next
  mIsDistinct = False
  mTop = ""
  mSelectList = ""
  mFrom = ""
  mWhere = ""
  mGroupBy = ""
  mHaving = ""
  mOrderBy = ""
End Sub


